
//---------------------------------------------------------------------------------------
//This code is for sunblide PCB with NPN transistors and buttons are operated by log 0
//The PCB is in box.
//Btn UP is on D6
//---------------------------------------------------------------------------------------

#include "ESPAsyncTCP.h"
#include "ESPAsyncWebServer.h"
#include "ESP8266WiFi.h"


#define RELAY_ON 1
#define RELAY_OFF 0
#define BTN_ON 0 //add constans due to different logic of NPN trans and buttons
#define BTN_OFF 1

// your WiFi credentials
const char* ssid = "xxxxxxxxx";
const char* password =  "xxxxxxxxxx";
AsyncWebServer server(80);

//53s for study room
int upDownTime = 63000; //full close or full open time - need to set for each blind in milliseconds
unsigned long startTimeUp = 0;
unsigned long endTimeUp = 0;
boolean timeRunningUp = false;
unsigned long startTimeDown = 0;
unsigned long endTimeDown = 0;
boolean timeRunningDown = false;

//delay constants between change of relay states
#define delayInterval_1 50 //drive manufacturer recommends 300ms
unsigned long time_1 = 0;

//inputs, outputs
const int relayPinUp = 5;//D1
const int relayPinDown = 4;//D2
const int btnDownInput = 2;//D4
const int btnUpInput = 12;//D6

int btnDownState = BTN_ON;
int btnUpState = BTN_ON;
int relayDownState = RELAY_OFF;
int relayUpState = RELAY_OFF;
int lastBtnDownState;
int lastBtnUpState;
int wifiInput = false;

unsigned long lastDebounceTime = 0;
unsigned long debounceDelay = 50;

unsigned long restartTime = 0;
int restartDelay = 10000;

String ip = "off";

void setup() {

  digitalWrite(relayPinUp, RELAY_OFF);
  digitalWrite(relayPinDown, RELAY_OFF);

  pinMode(relayPinUp, OUTPUT);
  pinMode(relayPinDown, OUTPUT);
  delay(2000);//check that all relays are inactive at reset

  pinMode(btnDownInput, INPUT_PULLUP);
  pinMode(btnUpInput, INPUT_PULLUP);

  Serial.begin(115200);
  WiFi.begin(ssid, password);

  //turn off broadcasting WiFi AP
  //ESP8266 with some firmware are set to broadcast WiFi AP
  WiFi.mode(WIFI_STA);

  Serial.println(WiFi.localIP());
  Serial.println(WiFi.macAddress());

  server.on("/up_off", HTTP_PUT, [](AsyncWebServerRequest * request) {
    request -> send(200, "text/plain", WiFi.localIP().toString());
    //digitalWrite(relayPinUp, RELAY_OFF);
    relayUpState = RELAY_OFF;
    wifiInput = false;
  });

  server.on("/up_on", HTTP_PUT, [](AsyncWebServerRequest * request) {
    request -> send(200, "text/plain", WiFi.localIP().toString());
    wifiInput = true;
    //digitalWrite(relayPinUp, RELAY_ON);
    relayUpState = RELAY_ON;
    time_1 = millis();
  });

  server.on("/down_off", HTTP_PUT, [](AsyncWebServerRequest * request) {
    request -> send(200, "text/plain", WiFi.localIP().toString());
    //digitalWrite(relayPinDown, RELAY_OFF);
    relayDownState = RELAY_OFF;
    wifiInput = false;
  });

  server.on("/down_on", HTTP_PUT, [](AsyncWebServerRequest * request) {
    request -> send(200, "text/plain", WiFi.localIP().toString());
    wifiInput = true;
    //digitalWrite(relayPinDown, RELAY_ON);
    relayDownState = RELAY_ON;
    time_1 = millis();
  });

  // sends status
  server.on("/status", HTTP_PUT, [](AsyncWebServerRequest * request) {
    request -> send(200, "text/plain", WiFi.localIP().toString());
  });

  // gets upDownTime in milisec
  server.on("/get_running_time", HTTP_PUT, [](AsyncWebServerRequest * request) {
    request -> send(200, "text/plain", String(upDownTime));
  });

  // sets upDownTime in milisec
  server.on("/set_running_time", HTTP_PUT, [](AsyncWebServerRequest * request) {
    request -> send(200, "text/plain", String(upDownTime));
  });

  server.begin();
}

void loop() {

  if (WiFi.status() == WL_CONNECTED) {
    if (ip != WiFi.localIP().toString()) {
      ip = WiFi.localIP().toString();
      Serial.println(ip);
    }
  }

  //reset counting for auto-off
  if (relayUpState == RELAY_OFF) {
    timeRunningUp = false;
  }
  if (relayDownState == RELAY_OFF) {
    timeRunningDown = false;
  }

  btnDownClick();
  btnUpClick();

  //auto-off after upDownTime
  if (relayUpState == RELAY_ON ) {
    if (durationTimeUp()) {
      relayUpState = RELAY_OFF;
    }
  }
  if (relayDownState == RELAY_ON) {
    if (durationTimeDown()) {
      relayDownState = RELAY_OFF;
    }
  }
}

void btnDownClick() {

  int readingbtnDown = digitalRead(btnDownInput);
  // check to see if you just pressed the button
  // (i.e. the input went from LOW to HIGH), and you've waited long enough
  // since the last press to ignore any noise:

  // If the switch changed, due to noise or pressing:
  if (readingbtnDown != lastBtnDownState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer than the debounce
    // delay, so take it as the actual current state:

    // if the button state has changed:
    if (readingbtnDown != btnDownState) {
      btnDownState = readingbtnDown;

      // only toggle the button if the new button state is HIGH
      if (btnDownState == BTN_ON) {
        relayUpState = RELAY_OFF;
        btnUpClick();
        relayDownState = !relayDownState;
        time_1 = millis();
      }
    }
  }
  // set the relay with delay:
  if (millis() > time_1 + delayInterval_1) {
    // set the relay:
    digitalWrite(relayPinDown, relayDownState);
  }


  // save the reading. Next time through the loop, it'll be the lastButtonState:
  lastBtnDownState = readingbtnDown;
}

void btnUpClick() {
  int readingBtnUp = digitalRead(btnUpInput);
  // check to see if you just pressed the button
  // (i.e. the input went from LOW to HIGH), and you've waited long enough
  // since the last press to ignore any noise:

  // If the switch changed, due to noise or pressing:
  if (readingBtnUp != lastBtnUpState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer than the debounce
    // delay, so take it as the actual current state:

    // if the button state has changed:
    if (readingBtnUp != btnUpState) {
      btnUpState = readingBtnUp;

      // only toggle the button if the new button state is HIGH
      if (btnUpState == BTN_ON) {
        relayDownState = RELAY_OFF;
        btnDownClick();
        relayUpState = !relayUpState;
        time_1 = millis();
      }
    }
  }
  // set the relay with delay:
  if (millis() > time_1 + delayInterval_1) {
    digitalWrite(relayPinUp, relayUpState);
  }



  // save the reading. Next time through the loop, it'll be the lastButtonState:
  lastBtnUpState = readingBtnUp;
}

boolean durationTimeUp() {
  if (timeRunningUp == false) {
    startTimeUp = millis();
    timeRunningUp = true;
  }
  if (timeRunningUp == true) {
    endTimeUp = millis();
    if ((endTimeUp - startTimeUp) > upDownTime) {
      timeRunningUp = false;
      return true;
    }
    else {
      return false;
    }
  }
}
boolean durationTimeDown() {
  if (timeRunningDown == false) {
    startTimeDown = millis();
    timeRunningDown = true;
  }
  if (timeRunningDown == true) {
    endTimeDown = millis();
    if ((endTimeDown - startTimeDown) > upDownTime) {
      timeRunningDown = false;
      return true;
    }
    else {
      return false;
    }
  }
}
